package org.jboss.aspects.observer;

import java.util.Collections;
import java.util.Iterator;
import java.util.HashSet;
import java.util.Set;

public abstract class Observing {

   protected interface Subject {
      void addObserver(Observer observer);
      void removeObserver(Observer observer);
      void notifyObservers();
   }

   protected interface Observer {
      void updateView(Subject s);
   }

   protected static class SubjectImpl implements Subject {

      private Set<Observer> observers = 
         Collections.synchronizedSet(new HashSet<Observer>());

      private Object owner;

      public SubjectImpl(Object owner) {
         this.owner = owner;
      }
   
      public void addObserver(Observer observer) {
         observers.add(observer);
      }
   
      public void removeObserver(Observer observer) {
         observers.remove(observer);
      }
   
      public void notifyObservers() {
         Subject subject = (Subject)owner;
         synchronized (observers) {
            for (Observer observer : observers)
               observer.updateView(subject);
         }
      }
   }

   public static void addObserver(Object subject, Object observer) {
      ((Subject)subject).addObserver((Observer)observer);
   }

   public static void removeObserver(Object subject, Object observer) {
      ((Subject)subject).removeObserver((Observer)observer);
   }

}
