package org.jboss.test.aop.bean;

import java.util.Iterator;
import java.util.List;

import org.jboss.aop.advice.Scope;
import org.jboss.aop.joinpoint.FieldInvocation;
import org.jboss.aop.joinpoint.Invocation;
import org.jboss.aop.Aspect;
import org.jboss.aop.Bind;
import org.jboss.aop.Mixin;

import org.jboss.aspects.observer.Observing;

@Aspect
public class LogUtilObserving extends Observing {

   private static class ObserverImpl implements Observer {

      private LogUtil ownerLogUtil;
      
      ObserverImpl(LogUtil ownerLogUtil) {
         this.ownerLogUtil = ownerLogUtil;
      }

      public void updateView(Subject subject) {
         ownerLogUtil.log("Received Notification: " + subject.toString());
      }
   }

   @Mixin(target=LogUtil.class, interfaces={Observing.Observer.class})
   public static ObserverImpl createObserverMixin(LogUtil logUtil) {
      return new ObserverImpl(logUtil);
   }

   @Mixin(target=Temperature.class, interfaces={Observing.Subject.class})
   public static SubjectImpl createSubjectMixin(Temperature temperature) {
      return new SubjectImpl(temperature);
   }

   @Bind(pointcut="set(* org.jboss.test.aop.bean.Temperature->*)")
   public Object changes(Invocation invocation) throws Throwable
   {
      FieldInvocation fi = (FieldInvocation) invocation;
      Object before = fi.getField().get(fi.getTargetObject());
      Object result = invocation.invokeNext();
      Object after = fi.getField().get(fi.getTargetObject());
      if (before == null && after != null || before.equals(after) == false) {
         Subject subject = (Subject)fi.getTargetObject();
         subject.notifyObservers();
      }
      return result;
   }

}
